@@ -63,12 +63,23 @@ static void
 				curconn->curbwin->blurb);
 			htmlstrip(buf);
 			secs_setvar("iftopic", buf);
+		} else if (curconn->curbwin->status != NULL) {
+			snprintf(buf, sizeof(buf), " (%.*s)",
+				(int)(sizeof(buf)-4),
+				curconn->curbwin->status);
+			htmlstrip(buf);
+			secs_setvar("iftopic", buf);
 		} else
 			secs_setvar("iftopic", "");
 
+		if ((curconn->curbwin->et != BUDDY) || (curconn->curbwin->e.buddy->typing == 0))
+			secs_setvar("iftyping", "");
+		else
+			secs_setvar("iftyping", secs_script_expand(NULL, getvar(curconn, "statusbar_typing")));
+
 		switch (curconn->curbwin->et) {
 		  case BUDDY:
-			if (curconn->curbwin->e.buddy->crypt != NULL)
+			if (curconn->curbwin->e.buddy->docrypt)
 				secs_setvar("ifcrypt", getvar(curconn, "statusbar_crypt"));
 			else
 				secs_setvar("ifcrypt", "");
@@ -104,6 +115,7 @@ static void
 		secs_setvar("cur", "");
 		secs_setvar("ifoper", "");
 		secs_setvar("ifquery", "");
+		secs_setvar("iftyping", "");
 		secs_setvar("ifchat", "");
 	}
 
@@ -133,6 +145,16 @@ static void
 		secs_setvar("iflag", "");
 	}
 
+	if (curconn->warnval > 0) {
+		snprintf(buf, sizeof(buf), "%li", curconn->warnval);
+		secs_setvar("warnval", buf);
+		secs_setvar("ifwarn",
+			secs_script_expand(NULL, getvar(curconn, "statusbar_warn")));
+	} else {
+		secs_setvar("warnval", "0");
+		secs_setvar("ifwarn", "");
+	}
+
 	if (strftime(buf, sizeof(buf), getvar(curconn, "statusbar"), tmptr) > 0) {
 		char	*left, *right;
 
@@ -522,8 +544,8 @@ buddywin_t
 		if (bwin->et == et) {
 			if (firetalk_compare_nicks(conn->conn, buddy, bwin->winname) == FE_SUCCESS)
 				return(bwin);
-			if ((bwin->et == BUDDY) && (firetalk_compare_nicks(conn->conn, buddy, USER_NAME(bwin->e.buddy)) == FE_SUCCESS))
-				return(bwin);
+//			if ((bwin->et == BUDDY) && (firetalk_compare_nicks(conn->conn, buddy, USER_NAME(bwin->e.buddy)) == FE_SUCCESS))
+//				return(bwin);
 		}
 	} while ((bwin = bwin->next) != conn->curbwin);
 
@@ -559,10 +581,9 @@ static void
 	}
 	free(bwin->pouncear);
 	bwin->pouncear = NULL;
-	free(bwin->winname);
-	bwin->winname = NULL;
-	free(bwin->blurb);
-	bwin->blurb = NULL;
+	FREESTR(bwin->winname);
+	FREESTR(bwin->blurb);
+	FREESTR(bwin->status);
 
 	if (bwin->nwin.logfile != NULL) {
 		struct tm	*tmptr;
@@ -767,7 +788,7 @@ void	playback(conn_t *const conn, buddywin_t *const bwin, const int lines) {
 	if ((rfile = playback_fopen(conn, bwin, "r")) != NULL) {
 		char	buf[2048];
 		int	maxlen = lines*faimconf.wstatus.widthx;
-		long	filesize, playbackstart, playbacklen;
+		long	filesize, playbackstart, playbacklen, pos;
 		time_t	lastprogress = now;
 
 #ifdef DEBUG_ECHO
@@ -777,23 +798,29 @@ void	playback(conn_t *const conn, buddywin_t *const bwin, const int lines) {
 		nw_statusbarf("Redrawing window for %s.", bwin->winname);
 
 		fseek(rfile, 0, SEEK_END);
-		filesize = ftell(rfile);
+		while (((filesize = ftell(rfile)) == -1) && (errno == EINTR))
+			;
+		assert(filesize >= 0);
 		if (filesize > maxlen) {
 			fseek(rfile, -maxlen, SEEK_CUR);
 			while ((fgetc(rfile) != '\n') && !feof(rfile))
 				;
 		} else
 			fseek(rfile, 0, SEEK_SET);
-		playbackstart = ftell(rfile);
+		while (((playbackstart = ftell(rfile)) == -1) && (errno == EINTR))
+			;
+		assert(playbackstart >= 0);
+		pos = 0;
 		playbacklen = filesize-playbackstart;
 		inplayback = 1;
 		while (fgets(buf, sizeof(buf), rfile) != NULL) {
-			while ((strlen(buf) > 0) && (buf[strlen(buf)-1] == '\n'))
-				buf[strlen(buf)-1] = 0;
+			long	len = strlen(buf);
+
+			pos += len;
 			hwprintf(&(bwin->nwin), -C(IMWIN,TEXT)-1, "%s", buf);
 			if ((now = time(NULL)) > lastprogress) {
 				nw_statusbarf("Redrawing window for %s (%li lines left).",
-					bwin->winname, lines*(playbacklen-(ftell(rfile)-playbackstart))/playbacklen);
+					bwin->winname, lines*(playbacklen-pos)/playbacklen);
 				lastprogress = now;
 			}
 		}
@@ -951,7 +978,7 @@ void	bgoing(conn_t *conn, const char *buddy) {
 		if ((blist->peer <= 0) && (blist->crypt != NULL))
 			echof(conn, NULL, "Strangeness while marking %s offline: no autopeer negotiated, but autocrypt set!\n",
 				buddy);
-		blist->peer = 0;
+		blist->docrypt = blist->peer = 0;
 		if (blist->crypt != NULL) {
 			free(blist->crypt);
 			blist->crypt = NULL;
@@ -964,7 +991,7 @@ void	bgoing(conn_t *conn, const char *buddy) {
 		status_echof(conn, "<font color=\"#00FFFF\">%s</font> <font color=\"#800000\">[<B>%s</B>]</font> has just logged off :(\n", 
 			user_name(NULL, 0, conn, blist), USER_GROUP(blist));
 		blist->offline = 1;
-		blist->isidle = blist->isaway = 0;
+		blist->warnval = blist->typing = blist->isidle = blist->isaway = 0;
 	} else
 		return;
 
@@ -978,10 +1005,8 @@ void	bgoing(conn_t *conn, const char *buddy) {
 				user_name(NULL, 0, conn, blist), USER_GROUP(blist));
 			if ((beeponsignon > 1) || ((awaytime == 0) && (beeponsignon == 1)))
 				beep();
-			if (bwin->blurb != NULL) {
-				free(bwin->blurb);
-				bwin->blurb = NULL;
-			}
+			FREESTR(bwin->blurb);
+			FREESTR(bwin->status);
 
 			if (bwin->keepafterso == 1) {
 				if ((autoclose > 0) && !USER_PERMANENT(bwin->e.buddy) && (bwin->waiting == 0))
@@ -1038,10 +1063,8 @@ void	baway(conn_t *conn, const char *buddy, int isaway) {
 	assert(blist != NULL);
 
 	if (bwin != NULL) {
-		if ((isaway == 0) && (bwin->blurb != NULL)) {
-			free(bwin->blurb);
-			bwin->blurb = NULL;
-		}
+		if (isaway == 0)
+			FREESTR(bwin->blurb);
 		if ((isaway == 1) && (blist->isaway == 0)) {
 			if ((conn->online+30) < now) {
 				awayc++;
@@ -1052,9 +1075,14 @@ void	baway(conn_t *conn, const char *buddy, int isaway) {
 			} else
 				window_echof(bwin, "<font color=\"#00FFFF\">%s</font> is now away.\n",
 					user_name(NULL, 0, conn, blist));
-		} else if ((isaway == 0) && (blist->isaway == 1))
-			window_echof(bwin, "<font color=\"#00FFFF\">%s</font> is no longer away!\n",
-				user_name(NULL, 0, conn, blist));
+		} else if ((isaway == 0) && (blist->isaway == 1)) {
+			if (bwin->status != NULL)
+				window_echof(bwin, "<font color=\"#00FFFF\">%s</font> is now available: %s\n",
+					user_name(NULL, 0, conn, blist), bwin->status);
+			else
+				window_echof(bwin, "<font color=\"#00FFFF\">%s</font> is no longer away!\n",
+					user_name(NULL, 0, conn, blist));
+		}
 	}
 
 	blist->isaway = isaway;
@@ -1062,10 +1090,8 @@ void	baway(conn_t *conn, const char *buddy, int isaway) {
 
 static void
 	bclearall_bwin(conn_t *conn, buddywin_t *bwin, int force) {
-	if (bwin->blurb != NULL) {
-		free(bwin->blurb);
-		bwin->blurb = NULL;
-	}
+	FREESTR(bwin->blurb);
+	FREESTR(bwin->status);
 	switch (bwin->et) {
 	  case BUDDY:
 		assert(bwin->e.buddy != NULL);
@@ -1107,7 +1133,7 @@ static void
 		free(buddy->tzname);
 		buddy->tzname = NULL;
 	}
-	buddy->peer = 0;
+	buddy->docrypt = buddy->peer = 0;
 	buddy->offline = 1;
 }
 
